<?php
/**
 * $Horde: framework/RPC/RPC/jsonrpc.php,v 1.2 2007/11/06 15:03:27 jan Exp $
 *
 * Copyright 2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @author  Joey Hewitt <joey@joeyhewitt.com>
 * @since   Horde 3.2
 * @package Horde_RPC
 */

/** Horde_Serialize */
require_once 'Horde/Serialize.php';

/**
 * The Horde_RPC_json-rpc class provides a JSONRPC implementation of the
 * Horde RPC system.  Based on the XMLRPC code.  The JSONRPC spec seems fuzzy
 * to me (though it's not like I implement things often), so I interpreted some
 * things as I thought they should be.  It looks like the 1.1 spec is
 * more clear, but it's still being ironed out.  I've copiusly commented to
 * help me clean up if I've messed something up. ;)
 *
 * @package Horde_RPC
 */
class Horde_RPC_jsonrpc extends Horde_RPC {

    /**
     * Set content type to text/javascript.
     *
     * @return string  JavaScript MIME type.
     */
    function getResponseContentType()
    {
        return 'application/json';
    }

    /**
     * Sends an RPC request to the server and returns the result.
     *
     * @param string  The raw request string.
     *
     * @return string  The JSON encoded response from the server.
     */
    function getResponse($request)
    {
        global $registry;

        /**
         * $request is Object (assoc. array) with
         *     method - String, method name
         *     params - Array, JS objects to pass to method
         *     id     - Request ID, used for matching responses
         * - http://json-rpc.org/wiki/specification#a1.1Requestmethodinvocation
         *
         * $reponse is associate array with
         *     result - Object returned by method, or null on error
         *     error  - Error object, if there was an error; null otherwise
         *     id     - the same ID from the request
         * - http://json-rpc.org/wiki/specification#a1.2Response
         */
        $request = (array)Horde_Serialize::unserialize($request, SERIALIZE_JSON);

        /**
         * I'm interpreting the following to mean $request may actually be an
         * array of requests:
         * "A client side peer may send one or more requests, notifications or
         * responses to its peer by sending an HTTP POST request containing
         * all serialized objects."
         * - http://json-rpc.org/wiki/specification#a2.2JSON-RPCoverHTTP
         */
        if (!is_array($request)) {
            $err = 'Request must be a JSON object';
        } elseif (isset($request[0]) && is_array($request[0])) {
            // I'm assuming requests and responses come in arrays...
            // Run sub-requests through and terminate now
            $responses = array();
            foreach ($request as $subreq) {
                $responses[] = $this->getResponse($subreq);
            }
            return Horde_Serialize($responses, SERIALIZE_JSON);
        }

        $response = array();
        $err = null;

        // Validate the request
        if (!$err) {
            if (!$request['method'] || !$request['params'] ||
                !isset($request['id'])) {
                $err = 'Request must be JSON and include method, params, and id properties';
            }
        }

        // Convert stdClasses to associative arrays, to match other RPC
        // protocols
        $request['params'] = $this->_classes2arrays($request['params']);

        // Check the method name
        // We'll assume the spec allows "." in method names...
        if (!$err) {
            $method = str_replace('.', '/', $request['method']);
            if (!$registry->hasMethod($method)) {
                $err = 'Method "' . $method . '" is not defined';
            }
        }

        // Call the method
        if (!$err) {
            $result = $registry->call($method, $request['params']);
            if (is_a($result, 'PEAR_Error')) {
                $err =
                    'PEAR_Error '.$result->getCode().': '.$result->getMessage();
                $result = null;
            }
        }

        // Return result/error. If error, wrap in JS Error object, using
        // class hinting
        // (http://json-rpc.org/wiki/specification#a3.JSONClasshinting)
        $response['id'] = $request['id'];
        if (!$err) {
            $response['result'] = $result;
            $response['error'] = null;
        } else {
            $response['result'] = null;
            $response['error'] = array('__jsonclass__' => array(
                'Error', array($err)
            ));
        }

        return Horde_Serialize::serialize($response, SERIALIZE_JSON);
    }

    /**
     * @ignore yes
     */
    function _classes2arrays($data)
    {
        if (is_a($data, 'stdClass')) {
            $data = get_object_vars($data);
        }
        if (is_array($data)) {
            foreach ($data as $key => $value) {
                $data[$key] = $this->_classes2arrays($value);
            }
        }
        return $data;
    }

}
